package com.zanxiang.sdk.service.Impl;

import com.alibaba.nacos.common.utils.MD5Utils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.domain.ResultVo;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.sdk.domain.dto.UserDTO;
import com.zanxiang.sdk.domain.entity.User;
import com.zanxiang.sdk.domain.enums.AccountStatusEnum;
import com.zanxiang.sdk.domain.enums.CmfSaltEnum;
import com.zanxiang.sdk.domain.params.LoginMobileParam;
import com.zanxiang.sdk.domain.params.LoginPasswordParam;
import com.zanxiang.sdk.domain.vo.UserLoginVO;
import com.zanxiang.sdk.mapper.UserMapper;
import com.zanxiang.sdk.service.IUserService;
import com.zanxiang.sdk.service.IUserTokenService;
import com.zanxiangnet.module.sms.service.impl.AliSmsService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.security.NoSuchAlgorithmException;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : 用户信息
 */
@Slf4j
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    @Autowired
    private AliSmsService aliSmsService;

    @Autowired
    private IUserTokenService userTokenService;

    /**
     * 用户名密码登录
     *
     * @param userLoginParam : 登录参数
     * @return : 返回登录token
     */
    @Override
    public ResultVo<UserLoginVO> loginPassword(LoginPasswordParam userLoginParam) {
        //用户名
        String username = userLoginParam.getUsername();
        //密码
        String password = userLoginParam.getPassword();
        //用户信息
        User user;
        //验证用户名是否为手机号
        if (StringUtils.checkPhone(username)) {
            user = super.getOne(new LambdaQueryWrapper<User>().eq(User::getMobile, username));
        } else {
            user = super.getOne(new LambdaQueryWrapper<User>().eq(User::getUsername, username));
        }
        //用户信息不存在
        if (user == null) {
            return new ResultVo<>(HttpStatusEnum.USERNAME_OR_PASSWORD_ERR);
        }
        //判断账号是否停用
        if (Objects.equals(AccountStatusEnum.FROZEN_STATUS.getStatus(), user.getStatus())) {
            return new ResultVo<>(HttpStatusEnum.ACCOUNT_HALT);
        }
        //验证密码
        if (!Objects.equals(this.cmfPassword(password), user.getPassword())) {
            return new ResultVo<>(HttpStatusEnum.USERNAME_OR_PASSWORD_ERR);
        }
        //验证通过, 获取token
        String userToken = userTokenService.getUserToken(user.getId(), user.getDeviceType());
        //构造返回
        return new ResultVo<>(new UserLoginVO(userToken));
    }


    /**
     * 手机号登录
     *
     * @param loginMobileParam : 手机号登录参数
     * @return : 返回登录信息
     */
    public ResultVo<UserLoginVO> loginMobile(LoginMobileParam loginMobileParam) {
        return null;
    }

    /**
     * 密码cfm加密
     *
     * @param password : 密码
     * @return : 返回加密完的密码
     */
    private String cmfPassword(String password) {
        String passwordSalt = password + CmfSaltEnum.CMF_PASSWORD.getSalt();
        try {
            return "###" + MD5Utils.md5Hex(MD5Utils.md5Hex(passwordSalt.getBytes()).getBytes());
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
            throw new BaseException("密码加密异常");
        }
    }

    /**
     * 检查用户手机号是否存在
     *
     * @param mobile : 用户手机号
     * @return : 返回用户信息
     */
    @Override
    public UserDTO getUserInfoByMobile(String mobile) {
        User user = super.getOne(new LambdaQueryWrapper<User>().eq(User::getMobile, mobile));
        return BeanUtils.copy(user, UserDTO.class);
    }

}
